@@ -1,54 +1,66 @@
-import numpy as np
-from ..Qt import QtCore, QtGui
-from ..Point import Point
+from math import atan2, degrees
+
 from .. import functions as fn
+from ..Point import Point
+from ..Qt import QtCore, QtGui, QtWidgets
 from .GraphicsObject import GraphicsObject
 
+__all__ = ['TextItem']
 
 class TextItem(GraphicsObject):
     """
     GraphicsItem displaying unscaled text (the text will always appear normal even inside a scaled ViewBox). 
     """
     def __init__(self, text='', color=(200,200,200), html=None, anchor=(0,0),
-                 border=None, fill=None, angle=0, rotateAxis=None):
+                 border=None, fill=None, angle=0, rotateAxis=None, ensureInBounds=False):
         """
-        ==============  =================================================================================
+        ================  =================================================================================
         **Arguments:**
-        *text*          The text to display
-        *color*         The color of the text (any format accepted by pg.mkColor)
-        *html*          If specified, this overrides both *text* and *color*
-        *anchor*        A QPointF or (x,y) sequence indicating what region of the text box will
-                        be anchored to the item's position. A value of (0,0) sets the upper-left corner
-                        of the text box to be at the position specified by setPos(), while a value of (1,1)
-                        sets the lower-right corner.
-        *border*        A pen to use when drawing the border
-        *fill*          A brush to use when filling within the border
-        *angle*         Angle in degrees to rotate text. Default is 0; text will be displayed upright.
-        *rotateAxis*    If None, then a text angle of 0 always points along the +x axis of the scene.
-                        If a QPointF or (x,y) sequence is given, then it represents a vector direction
-                        in the parent's coordinate system that the 0-degree line will be aligned to. This
-                        Allows text to follow both the position and orientation of its parent while still
-                        discarding any scale and shear factors.
-        ==============  =================================================================================
+        *text*            The text to display
+        *color*           The color of the text (any format accepted by pg.mkColor)
+        *html*            If specified, this overrides both *text* and *color*
+        *anchor*          A QPointF or (x,y) sequence indicating what region of the text box will
+                          be anchored to the item's position. A value of (0,0) sets the upper-left corner
+                          of the text box to be at the position specified by setPos(), while a value of (1,1)
+                          sets the lower-right corner.
+        *border*          A pen to use when drawing the border
+        *fill*            A brush to use when filling within the border
+        *angle*           Angle in degrees to rotate text. Default is 0; text will be displayed upright.
+        *rotateAxis*      If None, then a text angle of 0 always points along the +x axis of the scene.
+                          If a QPointF or (x,y) sequence is given, then it represents a vector direction
+                          in the parent's coordinate system that the 0-degree line will be aligned to. This
+                          Allows text to follow both the position and orientation of its parent while still
+                          discarding any scale and shear factors.
+        *ensureInBounds*  Ensures that the entire TextItem will be visible when using autorange, but may
+                          produce runaway scaling in certain circumstances (See issue #2642). Setting to 
+                          "True" retains legacy behavior.
+        ================  =================================================================================
 
 
         The effects of the `rotateAxis` and `angle` arguments are added independently. So for example:
 
-        * rotateAxis=None, angle=0 -> normal horizontal text
-        * rotateAxis=None, angle=90 -> normal vertical text
-        * rotateAxis=(1, 0), angle=0 -> text aligned with x axis of its parent
-        * rotateAxis=(0, 1), angle=0 -> text aligned with y axis of its parent
-        * rotateAxis=(1, 0), angle=90 -> text orthogonal to x axis of its parent        
+          * rotateAxis=None, angle=0 -> normal horizontal text
+          * rotateAxis=None, angle=90 -> normal vertical text
+          * rotateAxis=(1, 0), angle=0 -> text aligned with x axis of its parent
+          * rotateAxis=(0, 1), angle=0 -> text aligned with y axis of its parent
+          * rotateAxis=(1, 0), angle=90 -> text orthogonal to x axis of its parent
         """
                      
         self.anchor = Point(anchor)
         self.rotateAxis = None if rotateAxis is None else Point(rotateAxis)
         #self.angle = 0
         GraphicsObject.__init__(self)
-        self.textItem = QtGui.QGraphicsTextItem()
+        self.textItem = QtWidgets.QGraphicsTextItem()
         self.textItem.setParentItem(self)
         self._lastTransform = None
         self._lastScene = None
+
+        # Note: The following is pretty scuffed; ideally there would likely be 
+        # some inheritance changes, But this is the least-intrusive thing that 
+        # works for now
+        if ensureInBounds:
+            self.dataBounds = None
+        
         self._bounds = QtCore.QRectF()
         if html is None:
             self.setColor(color)
@@ -73,7 +85,7 @@ class TextItem(GraphicsObject):
         """
         Set the plain text to be rendered by this item. 
         
-        See QtGui.QGraphicsTextItem.setPlainText().
+        See QtWidgets.QGraphicsTextItem.setPlainText().
         """
         if text != self.toPlainText():
             self.textItem.setPlainText(text)
@@ -86,7 +98,7 @@ class TextItem(GraphicsObject):
         """
         Set the HTML code to be rendered by this item. 
         
-        See QtGui.QGraphicsTextItem.setHtml().
+        See QtWidgets.QGraphicsTextItem.setHtml().
         """
         if self.toHtml() != html:
             self.textItem.setHtml(html)
@@ -102,7 +114,7 @@ class TextItem(GraphicsObject):
         If the text requires more space than the width limit, then it will be
         wrapped into multiple lines.
         
-        See QtGui.QGraphicsTextItem.setTextWidth().
+        See QtWidgets.QGraphicsTextItem.setTextWidth().
         """
         self.textItem.setTextWidth(*args)
         self.updateTextPos()
@@ -111,7 +123,7 @@ class TextItem(GraphicsObject):
         """
         Set the font for this text. 
         
-        See QtGui.QGraphicsTextItem.setFont().
+        See QtWidgets.QGraphicsTextItem.setFont().
         """
         self.textItem.setFont(*args)
         self.updateTextPos()
@@ -135,7 +147,7 @@ class TextItem(GraphicsObject):
         """
         Set the color for this text.
         
-        See QtGui.QGraphicsItem.setDefaultTextColor().
+        See QtWidgets.QGraphicsItem.setDefaultTextColor().
         """
         self.color = fn.mkColor(color)
         self.textItem.setDefaultTextColor(self.color)
@@ -148,6 +160,18 @@ class TextItem(GraphicsObject):
         offset = (br - tl) * self.anchor
         self.textItem.setPos(-offset)
 
+    def dataBounds(self, ax, frac=1.0, orthoRange=None):
+        """
+        Returns only the anchor point for when calulating view ranges.
+        
+        Sacrifices some visual polish for fixing issue #2642.
+        """
+        if orthoRange:
+            range_min, range_max = orthoRange[0], orthoRange[1]
+            if not range_min <= self.anchor[ax] <= range_max:
+                return [None, None]
+
+        return [self.anchor[ax], self.anchor[ax]]
         
     def boundingRect(self):
         return self.textItem.mapRectToParent(self.textItem.boundingRect())
@@ -171,10 +195,10 @@ class TextItem(GraphicsObject):
             self.updateTransform()
             p.setTransform(self.sceneTransform())
         
-        if self.border.style() != QtCore.Qt.NoPen or self.fill.style() != QtCore.Qt.NoBrush:
+        if self.border.style() != QtCore.Qt.PenStyle.NoPen or self.fill.style() != QtCore.Qt.BrushStyle.NoBrush:
             p.setPen(self.border)
             p.setBrush(self.fill)
-            p.setRenderHint(p.Antialiasing, True)
+            p.setRenderHint(p.RenderHint.Antialiasing, True)
             p.drawPolygon(self.textItem.mapToParent(self.textItem.boundingRect()))
         
     def setVisible(self, v):
@@ -200,7 +224,7 @@ class TextItem(GraphicsObject):
         if not force and pt == self._lastTransform:
             return
 
-        t = pt.inverted()[0]
+        t = fn.invertQTransform(pt)
         # reset translation
         t.setMatrix(t.m11(), t.m12(), t.m13(), t.m21(), t.m22(), t.m23(), 0, 0, t.m33())
         
@@ -208,7 +232,7 @@ class TextItem(GraphicsObject):
         angle = -self.angle
         if self.rotateAxis is not None:
             d = pt.map(self.rotateAxis) - pt.map(Point(0, 0))
-            a = np.arctan2(d.y(), d.x()) * 180 / np.pi
+            a = degrees(atan2(d.y(), d.x()))
             angle += a
         t.rotate(angle)  
         self.setTransform(t)
